@@ -44,8 +44,6 @@
 
 static const char *backend_config_options(const struct Backend *);
 static uint32_t backend_regex_match_limit_for_len(size_t len);
-static int backend_cached_result(const struct Backend *, const char *, size_t, int *);
-static void backend_store_match_cache(struct Backend *, const char *, size_t, int);
 
 static uint32_t
 backend_regex_match_limit_for_len(size_t len) {
@@ -67,60 +65,6 @@ backend_regex_match_limit_for_len(size_t len) {
     return (uint32_t)limit;
 }
 
-static int
-backend_cached_result(const struct Backend *backend, const char *name, size_t len, int *matched) {
-    if (backend == NULL || !backend->last_lookup_valid)
-        return 0;
-
-    if (backend->last_lookup_len != len)
-        return 0;
-
-    if (len > 0) {
-        if (backend->last_lookup_name == NULL)
-            return 0;
-        if (memcmp(backend->last_lookup_name, name, len) != 0)
-            return 0;
-    }
-
-    if (matched != NULL)
-        *matched = backend->last_lookup_result;
-    return 1;
-}
-
-static void
-backend_store_match_cache(struct Backend *backend, const char *name, size_t len, int matched) {
-    if (backend == NULL)
-        return;
-
-    if (len > 0) {
-        if (len > SIZE_MAX - 1) {
-            backend->last_lookup_valid = 0;
-            return;
-        }
-        size_t needed = len + 1;
-        if (needed > backend->last_lookup_capacity) {
-            char *tmp = realloc(backend->last_lookup_name, needed);
-            if (tmp == NULL) {
-                backend->last_lookup_valid = 0;
-                return;
-            }
-            backend->last_lookup_name = tmp;
-            backend->last_lookup_capacity = needed;
-        }
-        if (backend->last_lookup_name != NULL && name != NULL) {
-            memcpy(backend->last_lookup_name, name, len);
-            backend->last_lookup_name[len] = '\0';
-        }
-    } else if (backend->last_lookup_name != NULL) {
-        backend->last_lookup_name[0] = '\0';
-    }
-
-    backend->last_lookup_len = len;
-    backend->last_lookup_result = matched ? 1 : 0;
-    backend->last_lookup_valid = 1;
-}
-
-
 static pcre2_match_context *backend_match_ctx;
 static int backend_match_ctx_registered;
 static void backend_regex_runtime_init(void);
@@ -142,11 +86,6 @@ new_backend(void) {
     backend->use_proxy_header = 0;
     backend->pattern_re = NULL;
     backend->pattern_match_data = NULL;
-    backend->last_lookup_name = NULL;
-    backend->last_lookup_len = 0;
-    backend->last_lookup_capacity = 0;
-    backend->last_lookup_result = 0;
-    backend->last_lookup_valid = 0;
 
     return backend;
 }
@@ -224,6 +163,9 @@ init_backend(struct Backend *backend) {
             return 0;
         }
 
+        pcre2_jit_compile(backend->pattern_re, PCRE2_JIT_COMPLETE);
+        /* JIT failure is non-fatal; pcre2_match falls back to interpreter */
+
         char address[ADDRESS_BUFFER_SIZE];
         debug("Parsed %s %s",
                 backend->pattern,
@@ -277,19 +219,12 @@ lookup_backend(const struct Backend_head *head, const char *name, size_t name_le
         if (iter->pattern_re == NULL)
             continue;
 
-        int cached_result;
-        if (backend_cached_result(iter, name, name_len, &cached_result)) {
-            if (cached_result)
-                return iter;
-            continue;
-        }
         pcre2_match_data *md = iter->pattern_match_data;
         if (md == NULL)
             continue;
 
         pcre2_set_match_limit(backend_match_ctx, match_limit);
         int ret = pcre2_match(iter->pattern_re, (const uint8_t *)name, name_len, 0, 0, md, backend_match_ctx);
-        backend_store_match_cache(iter, name, name_len, ret >= 0);
         if (ret >= 0)
             return iter;
     }
@@ -332,7 +267,6 @@ free_backend(struct Backend *backend) {
 
     free(backend->pattern);
     free(backend->address);
-    free(backend->last_lookup_name);
     if (backend->pattern_match_data != NULL)
         pcre2_match_data_free(backend->pattern_match_data);
     if (backend->pattern_re != NULL)
