@@ -124,8 +124,8 @@ buffer_pool_acquire(size_t size, int *pooled) {
         cls->head = next;
         if (cls->cached > 0)
             cls->cached--;
-        /* Use secure zeroing to prevent information disclosure */
-        secure_memzero(mem, cls->size);
+        /* Buffer was already zeroed on release; just clear pool metadata */
+        memset(mem, 0, sizeof(struct BufferPoolNode));
         if (pooled != NULL)
             *pooled = 1;
         return mem;
@@ -558,13 +558,9 @@ buffer_coalesce(struct Buffer *buffer, const void **dst) {
         memmove(buffer->buffer + first_len, buffer->buffer, second_len);
         memcpy(buffer->buffer, temp, first_len);
     } else {
-        if (first_len >= second_len * 2) {
-            memmove(buffer->buffer + second_len, buffer->buffer + head, first_len);
-        } else {
-            memcpy(temp, buffer->buffer, second_len);
-            memmove(buffer->buffer, buffer->buffer + head, first_len);
-            memcpy(buffer->buffer + first_len, temp, second_len);
-        }
+        memcpy(temp, buffer->buffer, second_len);
+        memmove(buffer->buffer, buffer->buffer + head, first_len);
+        memcpy(buffer->buffer + first_len, temp, second_len);
     }
 
     if (temp_on_heap)
