@@ -35,6 +35,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <netdb.h> /* getaddrinfo */
 #include <fcntl.h>
 #include <arpa/inet.h>
@@ -71,8 +72,9 @@
 #define SERVER_BUFFER_MIN_SIZE 32768
 #define SERVER_BUFFER_MAX_SIZE (1U << 20)
 #define BUFFER_SHRINK_IDLE_SECONDS 1.0
-#define CONNECTION_IDLE_TIMEOUT 60.0
-#define CONNECTION_HEADER_TIMEOUT 5.0
+
+static double connection_idle_timeout = CONNECTION_DEFAULT_IDLE_TIMEOUT;
+static double connection_header_timeout = CONNECTION_DEFAULT_HEADER_TIMEOUT;
 #define CONNECTION_MEMORY_PRESSURE_LIMIT (64U * 1024 * 1024)
 #define CONNECTION_MEMORY_PRESSURE_COOLDOWN 0.25
 #define DNS_CLIENT_BUCKET_BITS 10
@@ -148,6 +150,10 @@ static void shrink_candidate_update(struct Connection *, struct ev_loop *, ev_ts
 static void shrink_candidate_remove(struct Connection *);
 static void shrink_candidate_insert(struct Connection *);
 static ev_tstamp connection_last_activity(const struct Connection *);
+#ifdef SO_SPLICE
+static int try_splice(struct Connection *, struct ev_loop *);
+static void splice_cb(struct ev_loop *, struct ev_io *, int);
+#endif
 
 #define RATE_LIMIT_TABLE_SIZE 1024
 #define RATE_LIMIT_IDLE_TTL 300.0
@@ -309,6 +315,11 @@ accept_connection(struct Listener *listener, struct ev_loop *loop) {
     fcntl(sockfd, F_SETFL, flags | O_NONBLOCK);
 #endif
 
+    {
+        int on = 1;
+        setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on));
+    }
+
     if (!listener_acl_allows(listener, &con->client.addr)) {
         char addrbuf[INET6_ADDRSTRLEN];
         char listener_buf[ADDRESS_BUFFER_SIZE];
@@ -828,6 +839,16 @@ connection_cb(struct ev_loop *loop, struct ev_io *w, int revents) {
         server_open = server_socket_open(con);
     }
 
+#ifdef SO_SPLICE
+    /* Attempt kernel-level splice once both buffers have been flushed */
+    if (con->state == CONNECTED && !con->spliced &&
+            buffer_len(con->client.buffer) == 0 &&
+            buffer_len(con->server.buffer) == 0) {
+        if (try_splice(con, loop))
+            return;
+    }
+#endif
+
     /* Close other socket if we have flushed corresponding buffer */
     if (con->state == SERVER_CLOSED && buffer_len(con->server.buffer) == 0) {
         close_client_socket(con, loop);
@@ -905,7 +926,7 @@ reactivate_watcher(struct ev_loop *loop, struct ev_io *w,
     if (ev_is_active(w)) {
         if (events == 0)
             ev_io_stop(loop, w);
-        else if (events != w->events) {
+        else if (events != (w->events & (EV_READ | EV_WRITE))) {
             ev_io_stop(loop, w);
             ev_io_set(w, w->fd, events);
             ev_io_start(loop, w);
@@ -918,17 +939,17 @@ reactivate_watcher(struct ev_loop *loop, struct ev_io *w,
 
 static void
 reset_idle_timer_with_now(struct Connection *con, struct ev_loop *loop, ev_tstamp now) {
-    if (CONNECTION_IDLE_TIMEOUT <= 0.0)
+    if (connection_idle_timeout <= 0.0)
         return;
 
     if (ev_is_active(&con->idle_timer)) {
         double remaining = con->idle_timer.at - now;
-        if (remaining > CONNECTION_IDLE_TIMEOUT * 0.5)
+        if (remaining > connection_idle_timeout * 0.5)
             return;
         ev_timer_stop(loop, &con->idle_timer);
     }
 
-    ev_timer_set(&con->idle_timer, CONNECTION_IDLE_TIMEOUT, 0.0);
+    ev_timer_set(&con->idle_timer, connection_idle_timeout, 0.0);
     ev_timer_start(loop, &con->idle_timer);
 }
 
@@ -949,10 +970,10 @@ stop_idle_timer(struct Connection *con, struct ev_loop *loop) {
 
 static void
 start_header_timer(struct Connection *con, struct ev_loop *loop) {
-    if (CONNECTION_HEADER_TIMEOUT <= 0.0)
+    if (connection_header_timeout <= 0.0)
         return;
 
-    ev_timer_set(&con->header_timer, CONNECTION_HEADER_TIMEOUT, 0.0);
+    ev_timer_set(&con->header_timer, connection_header_timeout, 0.0);
     if (!ev_is_active(&con->header_timer))
         ev_timer_start(loop, &con->header_timer);
 }
@@ -1077,21 +1098,6 @@ hash_sockaddr_ip(const struct sockaddr_storage *addr, uint32_t *out_v4) {
     }
 }
 
-/* Constant-time memory comparison to prevent timing side-channel attacks.
- * Returns 1 if equal, 0 if not equal.
- * Execution time is independent of where differences occur. */
-static inline int
-constant_time_memcmp(const void *a, const void *b, size_t len) {
-    const unsigned char *pa = (const unsigned char *)a;
-    const unsigned char *pb = (const unsigned char *)b;
-    unsigned char diff = 0;
-
-    for (size_t i = 0; i < len; i++)
-        diff |= pa[i] ^ pb[i];
-
-    return diff == 0;
-}
-
 static int
 sockaddr_equal_ip(const struct sockaddr_storage *a, const struct sockaddr_storage *b) {
     if (a == b)
@@ -1102,9 +1108,7 @@ sockaddr_equal_ip(const struct sockaddr_storage *a, const struct sockaddr_storag
     if (a->ss_family == AF_INET) {
         const struct sockaddr_in *in_a = (const struct sockaddr_in *)a;
         const struct sockaddr_in *in_b = (const struct sockaddr_in *)b;
-        /* SECURITY: Use constant-time comparison to prevent timing attacks
-         * that could leak information about rate-limited IP addresses */
-        return constant_time_memcmp(&in_a->sin_addr, &in_b->sin_addr, sizeof(struct in_addr));
+        return memcmp(&in_a->sin_addr, &in_b->sin_addr, sizeof(struct in_addr)) == 0;
     } else if (a->ss_family == AF_INET6) {
         const struct sockaddr_in6 *in_a = (const struct sockaddr_in6 *)a;
         const struct sockaddr_in6 *in_b = (const struct sockaddr_in6 *)b;
@@ -1112,8 +1116,7 @@ sockaddr_equal_ip(const struct sockaddr_storage *a, const struct sockaddr_storag
         if (in_a->sin6_scope_id != in_b->sin6_scope_id)
             return 0;
 
-        /* SECURITY: Use constant-time comparison to prevent timing attacks */
-        return constant_time_memcmp(&in_a->sin6_addr, &in_b->sin6_addr, sizeof(struct in6_addr));
+        return memcmp(&in_a->sin6_addr, &in_b->sin6_addr, sizeof(struct in6_addr)) == 0;
     }
 
     return 0;
@@ -1417,6 +1420,30 @@ connections_set_global_limit(size_t limit) {
     max_global_connections = limit;
 }
 
+void
+connections_set_header_timeout(double timeout) {
+    if (timeout < 0.0)
+        timeout = 0.0;
+    connection_header_timeout = timeout;
+}
+
+double
+connections_get_header_timeout(void) {
+    return connection_header_timeout;
+}
+
+void
+connections_set_idle_timeout(double timeout) {
+    if (timeout < 0.0)
+        timeout = 0.0;
+    connection_idle_timeout = timeout;
+}
+
+double
+connections_get_idle_timeout(void) {
+    return connection_idle_timeout;
+}
+
 static void
 connection_memory_adjust(ssize_t delta) {
     if (delta >= 0) {
@@ -1480,7 +1507,14 @@ connection_idle_cb(struct ev_loop *loop, struct ev_timer *w, int revents __attri
 
     warn("Closing idle connection from %s after %.0f seconds without activity",
             display_sockaddr(&con->client.addr, con->client.addr_len, client, sizeof(client)),
-            CONNECTION_IDLE_TIMEOUT);
+            connection_idle_timeout);
+
+#ifdef SO_SPLICE
+    if (con->spliced) {
+        setsockopt(con->client.watcher.fd, SOL_SOCKET, SO_SPLICE, NULL, 0);
+        setsockopt(con->server.watcher.fd, SOL_SOCKET, SO_SPLICE, NULL, 0);
+    }
+#endif
 
     close_connection(con, loop);
     TAILQ_REMOVE(&connections, con, entries);
@@ -1501,7 +1535,7 @@ connection_header_timeout_cb(struct ev_loop *loop, struct ev_timer *w,
 
     warn("Closing connection from %s after %.0f seconds without initial request data",
             display_sockaddr(&con->client.addr, con->client.addr_len, client, sizeof(client)),
-            CONNECTION_HEADER_TIMEOUT);
+            connection_header_timeout);
 
     close_connection(con, loop);
     TAILQ_REMOVE(&connections, con, entries);
@@ -2132,6 +2166,11 @@ initiate_server_connect(struct Connection *con, struct ev_loop *loop) {
     fcntl(sockfd, F_SETFL, flags | O_NONBLOCK);
 #endif
 
+    {
+        int on = 1;
+        setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on));
+    }
+
     if (con->listener->transparent_proxy &&
             con->client.addr.ss_family == con->server.addr.ss_family) {
 #ifdef IP_TRANSPARENT
@@ -2184,23 +2223,9 @@ initiate_server_connect(struct Connection *con, struct ev_loop *loop) {
         }
     }
 
-    const int max_connect_retries = 3;
-    int attempt = 0;
-    int result;
-
-    do {
-        result = connect(sockfd,
-                (struct sockaddr *)&con->server.addr,
-                con->server.addr_len);
-        if (result == 0 || errno == EINPROGRESS)
-            break;
-        if (errno != EADDRNOTAVAIL || attempt >= max_connect_retries)
-            break;
-        attempt++;
-        struct timespec ts = { .tv_sec = 0, .tv_nsec = 25000000 };
-        while (nanosleep(&ts, &ts) < 0 && errno == EINTR)
-            ;
-    } while (1);
+    int result = connect(sockfd,
+            (struct sockaddr *)&con->server.addr,
+            con->server.addr_len);
 
     if (result < 0 && errno != EINPROGRESS) {
         close(sockfd);
@@ -2549,3 +2574,96 @@ print_connection(FILE *file, const struct Connection *con) {
             break;
     }
 }
+
+#ifdef SO_SPLICE
+/*
+ * Attempt to splice two connected sockets in the kernel.
+ * Returns 1 if splice was set up successfully (caller should return),
+ * or 0 if splice failed and user-space forwarding should continue.
+ */
+static int
+try_splice(struct Connection *con, struct ev_loop *loop) {
+    int client_fd = con->client.watcher.fd;
+    int server_fd = con->server.watcher.fd;
+
+    /* Set up splice with idle timeout matching the connection timeout */
+    struct splice sp;
+    memset(&sp, 0, sizeof(sp));
+    sp.sp_max = 0;  /* unlimited */
+
+    if (connection_idle_timeout > 0.0) {
+        sp.sp_idle.tv_sec = (time_t)connection_idle_timeout;
+        sp.sp_idle.tv_usec = (suseconds_t)
+            ((connection_idle_timeout - (double)sp.sp_idle.tv_sec) * 1e6);
+    }
+
+    /* Splice client → server */
+    sp.sp_fd = server_fd;
+    if (setsockopt(client_fd, SOL_SOCKET, SO_SPLICE, &sp, sizeof(sp)) < 0)
+        return 0;
+
+    /* Splice server → client */
+    sp.sp_fd = client_fd;
+    if (setsockopt(server_fd, SOL_SOCKET, SO_SPLICE, &sp, sizeof(sp)) < 0) {
+        /* Undo first splice */
+        if (setsockopt(client_fd, SOL_SOCKET, SO_SPLICE, NULL, 0) < 0)
+            warn("failed to unsplice client socket: %s", strerror(errno));
+        return 0;
+    }
+
+    con->spliced = 1;
+
+    /* Stop normal read/write watchers — kernel handles forwarding */
+    ev_io_stop(loop, &con->client.watcher);
+    ev_io_stop(loop, &con->server.watcher);
+
+    /* Watch both sockets for readability: when a splice ends (peer close,
+     * error, or idle timeout), the source socket becomes readable with 0
+     * bytes or an error. */
+    ev_io_init(&con->client.watcher, splice_cb, client_fd, EV_READ);
+    con->client.watcher.data = con;
+    ev_io_start(loop, &con->client.watcher);
+
+    ev_io_init(&con->server.watcher, splice_cb, server_fd, EV_READ);
+    con->server.watcher.data = con;
+    ev_io_start(loop, &con->server.watcher);
+
+    /* Keep the idle timer running as a safety net */
+    reset_idle_timer_with_now(con, loop, loop_now(loop));
+
+    return 1;
+}
+
+/*
+ * Callback for spliced connections.  Fires when splice terminates
+ * (peer close, error, or idle timeout).
+ */
+static void
+splice_cb(struct ev_loop *loop, struct ev_io *w, int revents __attribute__((unused))) {
+    struct Connection *con = (struct Connection *)w->data;
+
+    /* Unsplice both directions */
+    if (setsockopt(con->client.watcher.fd, SOL_SOCKET, SO_SPLICE, NULL, 0) < 0)
+        warn("failed to unsplice client socket: %s", strerror(errno));
+    if (setsockopt(con->server.watcher.fd, SOL_SOCKET, SO_SPLICE, NULL, 0) < 0)
+        warn("failed to unsplice server socket: %s", strerror(errno));
+
+    /* Stop splice watchers */
+    ev_io_stop(loop, &con->client.watcher);
+    ev_io_stop(loop, &con->server.watcher);
+
+    /* Close the connection — after splice we don't attempt to
+     * fall back to user-space forwarding since both directions
+     * are effectively finished. */
+    close_connection(con, loop);
+
+    TAILQ_REMOVE(&connections, con, entries);
+    connection_account_remove();
+
+    if (con->listener->access_log)
+        log_connection(con);
+
+    free_connection(con);
+    maybe_stop_buffer_shrink_timer(loop);
+}
+#endif
