@@ -44,8 +44,8 @@ struct Connection {
         RESOLVING,      /* DNS query in progress */
         RESOLVED,       /* Server socket address resolved */
         CONNECTED,      /* Connected to server */
-        SERVER_CLOSED,  /* Client closed socket */
-        CLIENT_CLOSED,  /* Server closed socket */
+        SERVER_CLOSED,  /* Server socket closed */
+        CLIENT_CLOSED,  /* Client socket closed */
         CLOSED          /* Both sockets closed */
     } state;
 
@@ -66,6 +66,9 @@ struct Connection {
     struct DnsClientUsageEntry *dns_client_usage;
     ev_tstamp established_timestamp;
     int use_proxy_header;
+#ifdef SO_SPLICE
+    int spliced;
+#endif
 
     TAILQ_ENTRY(Connection) entries;
     TAILQ_ENTRY(Connection) shrink_entries;
@@ -91,4 +94,12 @@ void connections_set_dns_query_per_client_limit(size_t limit);
 void connections_set_buffer_limits(size_t client_limit, size_t server_limit);
 void connections_set_global_limit(size_t limit);
 
+#define CONNECTION_DEFAULT_HEADER_TIMEOUT 5.0
+#define CONNECTION_DEFAULT_IDLE_TIMEOUT 60.0
+
+void connections_set_header_timeout(double timeout);
+double connections_get_header_timeout(void);
+void connections_set_idle_timeout(double timeout);
+double connections_get_idle_timeout(void);
+
 #endif
