@@ -37,6 +37,59 @@
 #define SERVER_NAME_LEN 256
 #define HTTP2_DEFAULT_DYNAMIC_TABLE_SIZE 4096
 
+static size_t http2_max_headers = HTTP2_DEFAULT_MAX_HEADERS;
+static size_t http2_max_frame_size = HTTP2_DEFAULT_MAX_FRAME_SIZE;
+static size_t http2_max_frames_per_connection = HTTP2_DEFAULT_MAX_FRAMES_PER_CONNECTION;
+static size_t http2_max_continuation_frames = HTTP2_DEFAULT_MAX_CONTINUATION_FRAMES;
+
+size_t
+http2_get_max_headers(void) {
+    return http2_max_headers;
+}
+
+void
+http2_set_max_headers(size_t max_headers) {
+    if (max_headers == 0)
+        max_headers = 1;
+    http2_max_headers = max_headers;
+}
+
+size_t
+http2_get_max_frame_size(void) {
+    return http2_max_frame_size;
+}
+
+void
+http2_set_max_frame_size(size_t max_size) {
+    if (max_size == 0)
+        max_size = 1;
+    http2_max_frame_size = max_size;
+}
+
+size_t
+http2_get_max_frames_per_connection(void) {
+    return http2_max_frames_per_connection;
+}
+
+void
+http2_set_max_frames_per_connection(size_t max_frames) {
+    if (max_frames == 0)
+        max_frames = 1;
+    http2_max_frames_per_connection = max_frames;
+}
+
+size_t
+http2_get_max_continuation_frames(void) {
+    return http2_max_continuation_frames;
+}
+
+void
+http2_set_max_continuation_frames(size_t max_frames) {
+    if (max_frames == 0)
+        max_frames = 1;
+    http2_max_continuation_frames = max_frames;
+}
+
 struct hpack_entry {
     char *name;
     size_t name_len;
@@ -49,6 +102,7 @@ struct hpack_decoder {
     struct hpack_entry *dynamic_entries;
     size_t dynamic_count;
     size_t dynamic_capacity;
+    size_t dynamic_head;      /* ring buffer start index */
     size_t dynamic_size;
     size_t max_dynamic_size;
 };
@@ -314,7 +368,8 @@ hpack_drop_last_entry(struct hpack_decoder *decoder) {
     if (decoder == NULL || decoder->dynamic_count == 0)
         return;
 
-    struct hpack_entry *entry = &decoder->dynamic_entries[decoder->dynamic_count - 1];
+    size_t tail = (decoder->dynamic_head + decoder->dynamic_count - 1) % decoder->dynamic_capacity;
+    struct hpack_entry *entry = &decoder->dynamic_entries[tail];
 
     if (decoder->dynamic_size >= entry->total_size)
         decoder->dynamic_size -= entry->total_size;
@@ -371,7 +426,7 @@ parse_frames(const unsigned char *data, size_t data_len, char **hostname) {
         if (remaining < 9)
             break;
 
-        if (++frame_count > HTTP2_MAX_FRAMES_PER_CONNECTION) {
+        if (++frame_count > http2_max_frames_per_connection) {
             result = -4;
             goto done;
         }
@@ -387,7 +442,7 @@ parse_frames(const unsigned char *data, size_t data_len, char **hostname) {
                              data[pos + 8];
         pos += 9;
 
-        if (length > HTTP2_MAX_FRAME_SIZE) {
+        if (length > http2_max_frame_size) {
             result = -4;
             goto done;
         }
@@ -464,7 +519,7 @@ parse_frames(const unsigned char *data, size_t data_len, char **hostname) {
                     result = -4;
                     goto done;
                 }
-                if (++block.continuation_count > HTTP2_MAX_CONTINUATION_FRAMES) {
+                if (++block.continuation_count > http2_max_continuation_frames) {
                     result = -4;
                     goto done;
                 }
@@ -537,10 +592,17 @@ decode_header_block(struct hpack_decoder *decoder,
         struct host_accumulator *hosts) {
     size_t pos = 0;
     size_t decoded_budget = HTTP2_MAX_HEADER_BLOCK_SIZE;
+    size_t header_count = 0;
 
     while (pos < len) {
         unsigned char byte = data[pos];
 
+        /* Dynamic table size updates don't count as headers */
+        if ((byte & 0xE0) != 0x20) {
+            if (++header_count > http2_max_headers)
+                return -4;
+        }
+
         if (byte & 0x80) { /* Indexed header field */
             size_t index;
             size_t consumed;
@@ -753,6 +815,7 @@ hpack_decoder_init(struct hpack_decoder *decoder) {
     decoder->dynamic_entries = NULL;
     decoder->dynamic_count = 0;
     decoder->dynamic_capacity = 0;
+    decoder->dynamic_head = 0;
     decoder->dynamic_size = 0;
     decoder->max_dynamic_size = HTTP2_DEFAULT_DYNAMIC_TABLE_SIZE;
 }
@@ -768,6 +831,7 @@ hpack_decoder_free(struct hpack_decoder *decoder) {
     free(decoder->dynamic_entries);
     decoder->dynamic_entries = NULL;
     decoder->dynamic_capacity = 0;
+    decoder->dynamic_head = 0;
     decoder->dynamic_size = 0;
 }
 
@@ -797,7 +861,6 @@ hpack_add_entry(struct hpack_decoder *decoder, const char *name, size_t name_len
 
     size_t entry_size = combined_len + 32;
     assert(entry_size >= combined_len && entry_size >= 32);
-    int reserved = 0;
 
     if (entry_size > decoder->max_dynamic_size) {
         while (decoder->dynamic_count > 0)
@@ -820,43 +883,50 @@ hpack_add_entry(struct hpack_decoder *decoder, const char *name, size_t name_len
             return 0;
 
         size_t alloc_size = new_cap * sizeof(struct hpack_entry);
-        struct hpack_entry *tmp = realloc(decoder->dynamic_entries, alloc_size);
+        struct hpack_entry *tmp = malloc(alloc_size);
         if (tmp == NULL)
             return 0;
+
+        /* Linearize ring buffer into new allocation */
+        for (size_t i = 0; i < decoder->dynamic_count; i++) {
+            size_t src = (decoder->dynamic_head + i) % decoder->dynamic_capacity;
+            tmp[i] = decoder->dynamic_entries[src];
+        }
+        free(decoder->dynamic_entries);
         decoder->dynamic_entries = tmp;
+        decoder->dynamic_head = 0;
         decoder->dynamic_capacity = new_cap;
     }
 
+    if (name_len > SIZE_MAX - 1 || value_len > SIZE_MAX - 1)
+        return 0;
+
     if (!hpack_global_try_reserve(entry_size))
         return 0;
-    reserved = 1;
 
-    memmove(&decoder->dynamic_entries[1], &decoder->dynamic_entries[0], decoder->dynamic_count * sizeof(struct hpack_entry));
-
-    struct hpack_entry *entry = &decoder->dynamic_entries[0];
-    if (name_len > SIZE_MAX - 1 || value_len > SIZE_MAX - 1) {
-        memmove(&decoder->dynamic_entries[0], &decoder->dynamic_entries[1], decoder->dynamic_count * sizeof(struct hpack_entry));
-        if (reserved)
-            hpack_global_release(entry_size);
+    char *new_name = malloc(name_len + 1);
+    char *new_value = malloc(value_len + 1);
+    if (new_name == NULL || new_value == NULL) {
+        free(new_name);
+        free(new_value);
+        hpack_global_release(entry_size);
         return 0;
     }
 
-    entry->name = malloc(name_len + 1);
-    entry->value = malloc(value_len + 1);
-    if ((name_len && entry->name == NULL) || (value_len && entry->value == NULL)) {
-        free(entry->name);
-        free(entry->value);
-        memmove(&decoder->dynamic_entries[0], &decoder->dynamic_entries[1], decoder->dynamic_count * sizeof(struct hpack_entry));
-        if (reserved)
-            hpack_global_release(entry_size);
-        return 0;
-    }
+    memcpy(new_name, name, name_len);
+    new_name[name_len] = '\0';
+    memcpy(new_value, value, value_len);
+    new_value[value_len] = '\0';
 
-    memcpy(entry->name, name, name_len);
-    entry->name[name_len] = '\0';
+    /* Insert at front of ring buffer */
+    decoder->dynamic_head = decoder->dynamic_head == 0
+        ? decoder->dynamic_capacity - 1
+        : decoder->dynamic_head - 1;
+
+    struct hpack_entry *entry = &decoder->dynamic_entries[decoder->dynamic_head];
+    entry->name = new_name;
     entry->name_len = name_len;
-    memcpy(entry->value, value, value_len);
-    entry->value[value_len] = '\0';
+    entry->value = new_value;
     entry->value_len = value_len;
     entry->total_size = entry_size;
 
@@ -890,7 +960,8 @@ hpack_get_indexed(const struct hpack_decoder *decoder, size_t index,
     if (dynamic_index >= decoder->dynamic_count)
         return 0;
 
-    const struct hpack_entry *entry = &decoder->dynamic_entries[dynamic_index];
+    size_t ring_index = (decoder->dynamic_head + dynamic_index) % decoder->dynamic_capacity;
+    const struct hpack_entry *entry = &decoder->dynamic_entries[ring_index];
     if (name != NULL) {
         *name = entry->name;
         *name_len = entry->name_len;
