@@ -68,6 +68,13 @@ static uint64_t host_to_be64(uint64_t host) {
  * the counter threshold is never reached. */
 #define IPC_CRYPTO_REKEY_INTERVAL (7 * 24 * 60 * 60)  /* 7 days in seconds */
 
+/* Maximum allowed generation gap between sender and receiver.
+ * This limits DoS attacks where an attacker sends a message with a
+ * very large generation number, forcing the receiver to perform
+ * billions of rekey operations. A gap of 16 allows for reasonable
+ * message reordering and recovery scenarios while preventing abuse. */
+#define IPC_CRYPTO_MAX_GENERATION_GAP 16
+
 /* HKDF-SHA256 key derivation function.
  * This replaces raw SHA256 hashing with proper HKDF as per RFC 5869.
  * salt: optional salt value (can be NULL)
@@ -641,6 +648,19 @@ ipc_crypto_open(struct ipc_crypto_state *state, const uint8_t *frame,
         return -1;
     }
 
+    /* Reject excessive generation gap to prevent DoS via forced rekey loop */
+    if (msg_generation - state->recv_generation > IPC_CRYPTO_MAX_GENERATION_GAP) {
+        EVP_CIPHER_CTX_free(ctx);
+        OPENSSL_cleanse(output, payload_len > 0 ? payload_len : 1);
+        free(output);
+        warn("IPC generation gap too large: msg_generation=%u, recv_generation=%u, gap=%u",
+                msg_generation, state->recv_generation,
+                msg_generation - state->recv_generation);
+        ipc_crypto_mask_failure(payload_len);
+        errno = EBADMSG;
+        return -1;
+    }
+
     /* Advance recv_generation to match sender if needed */
     while (state->recv_generation < msg_generation) {
         if (ipc_crypto_rekey_recv(state) < 0) {
@@ -828,24 +848,20 @@ ipc_crypto_recv_msg(struct ipc_crypto_state *state, int sockfd,
         memcpy(&fd_received, CMSG_DATA(cmsg), sizeof(int));
     }
 
+    if (ipc_crypto_open(state, cipher, frame_len, max_payload_len,
+                plaintext, plaintext_len) < 0) {
+        free(cipher);
+        if (fd_received >= 0)
+            close(fd_received);
+        errno = EBADMSG;
+        return -1;
+    }
+
     if (received_fd != NULL)
         *received_fd = fd_received;
     else if (fd_received >= 0)
         close(fd_received);
 
-    if (ipc_crypto_open(state, cipher, frame_len, max_payload_len,
-                plaintext, plaintext_len) < 0) {
-        free(cipher);
-        if (received_fd != NULL && fd_received >= 0) {
-            close(fd_received);
-            *received_fd = -1;
-        } else if (received_fd == NULL && fd_received >= 0) {
-            close(fd_received);
-        }
-        errno = EBADMSG;
-        return -1;
-    }
-
     free(cipher);
     return 1;
 }
