@@ -48,6 +48,7 @@
 #include "protocol.h"
 #include "tls.h"
 #include "http.h"
+#include "xmpp.h"
 #include "fd_util.h"
 
 #define LISTENER_ACCEPT_MAX_BATCH 64
@@ -289,7 +290,12 @@ listeners_reload(struct Listener_head *existing_listeners,
              * config */
             SLIST_REMOVE(new_listeners, new_listener, Listener, entries);
             add_listener(existing_listeners, new_listener);
-            init_listener(new_listener, tables, loop);
+            if (init_listener(new_listener, tables, loop) < 0) {
+                err("Failed to initialize listener %s",
+                        display_address(new_listener->address,
+                                address, sizeof(address)));
+                remove_listener(existing_listeners, new_listener, loop);
+            }
 
             /* -1 for removing from new_listeners */
             listener_ref_put(new_listener);
@@ -431,6 +437,7 @@ accept_listener_arg(struct Listener *listener, const char *arg) {
         }
     } else {
         err("Invalid listener argument %s", arg);
+        return -1;
     }
 
     return 1;
@@ -455,6 +462,8 @@ int
 accept_listener_protocol(struct Listener *listener, const char *protocol) {
     if (strncasecmp(protocol, http_protocol->name, strlen(protocol)) == 0)
         listener->protocol = http_protocol;
+    else if (strncasecmp(protocol, xmpp_protocol->name, strlen(protocol)) == 0)
+        listener->protocol = xmpp_protocol;
     else
         listener->protocol = tls_protocol;
 
@@ -647,7 +656,8 @@ valid_listener(const struct Listener *listener) {
             return 0;
     }
 
-    if (listener->protocol != tls_protocol && listener->protocol != http_protocol) {
+    if (listener->protocol != tls_protocol && listener->protocol != http_protocol &&
+            listener->protocol != xmpp_protocol) {
         err("Invalid protocol");
         return 0;
     }
