@@ -10,7 +10,7 @@ FUZZ_RUNTIME=${FUZZ_RUNTIME:-30}
 FUZZ_VERBOSE=${FUZZ_VERBOSE:-1}
 FUZZ_PARALLEL=${FUZZ_PARALLEL:-0}
 EXTRA_FLAGS=${FUZZ_CFLAGS:-"-O1 -g"}
-COMMON_FLAGS=("-fsanitize=fuzzer,address,undefined" "-fno-omit-frame-pointer" "-DFUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION" "-DHAVE_CONFIG_H" "-DHAVE_LIBPCRE2_8" "-I$ROOT_DIR" "-I$ROOT_DIR/src")
+COMMON_FLAGS=("-fsanitize=fuzzer,address,undefined" "-fno-omit-frame-pointer" "-fno-sanitize-address-use-odr-indicator" "-DFUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION" "-DHAVE_CONFIG_H" "-DHAVE_LIBPCRE2_8" "-I$ROOT_DIR" "-I$ROOT_DIR/src")
 
 if command -v pkg-config >/dev/null 2>&1 && pkg-config --exists libpcre2-8; then
     : "${PCRE2_CFLAGS:=$(pkg-config --cflags libpcre2-8)}"
@@ -77,6 +77,7 @@ mkdir -p "$OUT_DIR" \
     "$CORPUS_ROOT/tls" \
     "$CORPUS_ROOT/http2" \
     "$CORPUS_ROOT/http" \
+    "$CORPUS_ROOT/xmpp" \
     "$CORPUS_ROOT/hostname" \
     "$CORPUS_ROOT/cfg_tokenizer" \
     "$CORPUS_ROOT/ipc_crypto" \
@@ -145,6 +146,11 @@ build_fuzzer listener_acl_fuzz \
     "$ROOT_DIR/src/address.c" \
     "$ROOT_DIR/src/backend.c" \
     "$ROOT_DIR/src/table.c" \
+    "$ROOT_DIR/src/tls.c" \
+    "$ROOT_DIR/src/http.c" \
+    "$ROOT_DIR/src/http2.c" \
+    "$ROOT_DIR/src/http2_huffman.c" \
+    "$ROOT_DIR/src/xmpp.c" \
     -I"$ROOT_DIR/tests/include"
 
 build_fuzzer resolver_response_fuzz \
@@ -175,6 +181,7 @@ build_fuzzer config_fuzz \
     "$ROOT_DIR/src/http.c" \
     "$ROOT_DIR/src/http2.c" \
     "$ROOT_DIR/src/http2_huffman.c" \
+    "$ROOT_DIR/src/xmpp.c" \
     "$ROOT_DIR/src/seccomp.c" \
     -lev -lssl -lcrypto -lcares -lseccomp
 
@@ -200,6 +207,10 @@ build_fuzzer tls_fuzz \
     "$ROOT_DIR/tests/fuzz/tls_fuzz.c" \
     "$ROOT_DIR/src/tls.c"
 
+build_fuzzer xmpp_fuzz \
+    "$ROOT_DIR/tests/fuzz/xmpp_fuzz.c" \
+    "$ROOT_DIR/src/xmpp.c"
+
 vlog "Fuzzers built successfully."
 
 if [[ ${RUN_FUZZ:-1} -eq 0 ]]; then
@@ -218,6 +229,7 @@ FUZZ_TARGETS=(
     "config_fuzz:config"
     "http2_fuzz:http2"
     "http_fuzz:http"
+    "xmpp_fuzz:xmpp"
     "hostname_fuzz:hostname"
     "cfg_tokenizer_fuzz:cfg_tokenizer"
     "tls_fuzz:tls"
